@@ -354,7 +354,7 @@ void PeerWireClient::timerEvent(QTimerEvent *event)
     if (event->timerId() == transferSpeedTimer) {
         // Rotate the upload / download records.
         for (int i = 6; i >= 0; --i) {
-            uploadSpeedData[i + i] = uploadSpeedData[i];
+            uploadSpeedData[i + 1] = uploadSpeedData[i];
             downloadSpeedData[i + 1] = downloadSpeedData[i];
         }
         uploadSpeedData[0] = 0;
