@@ -26,13 +26,13 @@
 
 #include <stddef.h>
 
-#define QT_VERSION_STR   "4.1.2"
+#define QT_VERSION_STR   "4.1.3"
 /*
    QT_VERSION is (major << 16) + (minor << 8) + patch.
 */
-#define QT_VERSION 0x040102
+#define QT_VERSION 0x040103
 
-#define QT_PACKAGEDATE_STR "2006-03-20"
+#define QT_PACKAGEDATE_STR "2006-05-14"
 
 #if !defined(QT_BUILD_MOC)
 #include <QtCore/qconfig.h>
@@ -256,7 +256,6 @@
 /* can we just compare the structures? */
 #    define Q_FULL_TEMPLATE_INSTANTIATION_MEMCMP
 /* these are not useful to our customers */
-#    define QT_QWS_NO_SHM
 #    define QT_NO_QWS_MULTIPROCESS
 #    define QT_NO_QWS_CURSOR
 #  endif
@@ -267,31 +266,30 @@
 #  if defined(__INTEL_COMPILER)
 /* Intel C++ also masquerades as GCC 3.2.0 */
 #    define Q_CC_INTEL
-#  else
-#    ifdef __APPLE__
-#      define Q_NO_DEPRECATED_CONSTRUCTORS
-#    endif
-#    if __GNUC__ == 2 && __GNUC_MINOR__ <= 7
-#      define Q_FULL_TEMPLATE_INSTANTIATION
-#    endif
+#  endif
+#  ifdef __APPLE__
+#    define Q_NO_DEPRECATED_CONSTRUCTORS
+#  endif
+#  if __GNUC__ == 2 && __GNUC_MINOR__ <= 7
+#    define Q_FULL_TEMPLATE_INSTANTIATION
+#  endif
 /* GCC 2.95 knows "using" but does not support it correctly */
-#    if __GNUC__ == 2 && __GNUC_MINOR__ <= 95
-#      define Q_NO_USING_KEYWORD
-#      define QT_NO_STL_WCHAR
-#    endif
+#  if __GNUC__ == 2 && __GNUC_MINOR__ <= 95
+#    define Q_NO_USING_KEYWORD
+#    define QT_NO_STL_WCHAR
+#  endif
 /* GCC 3.1 and GCC 3.2 wrongly define _SB_CTYPE_MACROS on HP-UX */
-#    if defined(Q_OS_HPUX) && __GNUC__ == 3 && __GNUC_MINOR__ >= 1
-#      define Q_WRONG_SB_CTYPE_MACROS
-#    endif
+#  if defined(Q_OS_HPUX) && __GNUC__ == 3 && __GNUC_MINOR__ >= 1
+#    define Q_WRONG_SB_CTYPE_MACROS
+#  endif
 /* Apple's GCC 3.1 chokes on our streaming qDebug() */
-#    if defined(Q_OS_DARWIN) && __GNUC__ == 3 && (__GNUC_MINOR__ >= 1 && __GNUC_MINOR__ < 3)
-#      define Q_BROKEN_DEBUG_STREAM
-#    endif
-#    if (defined(__arm__) || defined(__ARMEL__)) && !defined(QT_MOC_CPP)
-#      define Q_PACKED __attribute__ ((__packed__))
-#      define Q_NO_PACKED_REFERENCE
-#    endif
-#  endif /* __INTEL_COMPILER */
+#  if defined(Q_OS_DARWIN) && __GNUC__ == 3 && (__GNUC_MINOR__ >= 1 && __GNUC_MINOR__ < 3)
+#    define Q_BROKEN_DEBUG_STREAM
+#  endif
+#  if (defined(__arm__) || defined(__ARMEL__)) && !defined(QT_MOC_CPP)
+#    define Q_PACKED __attribute__ ((__packed__))
+#    define Q_NO_PACKED_REFERENCE
+#  endif
 
 /* IBM compiler versions are a bit messy. There are actually two products:
    the C product, and the C++ product. The C++ compiler is always packaged
@@ -511,7 +509,7 @@
 
      MACX     - Mac OS X
      MAC9     - Mac OS 9
-     QWS      - Qt/Embedded
+     QWS      - Qtopia Core
      WIN32    - Windows
      X11      - X Window System
      PM       - unsupported
@@ -607,6 +605,15 @@ typedef unsigned long ulong;
 #define FALSE false
 #endif
 
+#if defined(Q_OS_MAC)
+#define QT_BEGIN_HEADER extern "C++" {
+#define QT_END_HEADER }
+#else
+#define QT_BEGIN_HEADER
+#define QT_END_HEADER
+#endif
+
+QT_BEGIN_HEADER
 /*
    Proper for-scoping in VC++6 and MIPSpro CC
 */
@@ -1780,6 +1787,8 @@ QT_LICENSED_MODULE(ActiveQt)
 #define QT_MODULE(x) \
     typedef QtValidLicenseFor##x##Module Qt##x##Module;
 
+QT_END_HEADER
+
 #endif /* __cplusplus */
 
 #endif /* QGLOBAL_H */
