@@ -27,6 +27,8 @@
 #include <QtCore/qstring.h>
 #include <QtCore/qnamespace.h>
 
+QT_BEGIN_HEADER
+
 QT_MODULE(Core)
 
 class Q_CORE_EXPORT QDate
@@ -279,4 +281,6 @@ Q_CORE_EXPORT QDebug operator<<(QDebug, const QTime &);
 Q_CORE_EXPORT QDebug operator<<(QDebug, const QDateTime &);
 #endif
 
+QT_END_HEADER
+
 #endif // QDATETIME_H
