@@ -26,6 +26,8 @@
 
 #include <QtCore/qlist.h>
 
+QT_BEGIN_HEADER
+
 QT_MODULE(Core)
 
 template <class T>
@@ -40,4 +42,6 @@ public:
     inline const T &head() const { return QList<T>::first(); }
 };
 
+QT_END_HEADER
+
 #endif // QQUEUE_H
