@@ -55,6 +55,8 @@ typedef std::basic_string<wchar_t> QStdWString;
 #error qstring.h must be included before any header file that defines truncate
 #endif
 
+QT_BEGIN_HEADER
+
 QT_MODULE(Core)
 
 class QCharRef;
@@ -803,6 +805,13 @@ inline QStdWString QString::toStdWString() const
 {
     QStdWString str;
     str.resize(length());
+
+#if defined(_MSC_VER) && _MSC_VER >= 1400
+    // VS2005 crashes if the string is empty
+    if (!length())
+        return str;
+#endif
+
     str.resize(toWCharArray(&(*str.begin())));
     return str;
 }
@@ -845,4 +854,6 @@ extern Q_CORE_EXPORT QByteArray qt_winQString2MB(const QChar *ch, int len);
 extern Q_CORE_EXPORT QString qt_winMB2QString(const char* mb, int len=-1);
 #endif
 
+QT_END_HEADER
+
 #endif // QSTRING_H
