@@ -319,9 +319,6 @@ void QWellArray::setCurrent(int row, int col)
 
 void QWellArray::setSelected(int row, int col)
 {
-    if ((selRow == row) && (selCol == col))
-        return;
-
     int oldRow = selRow;
     int oldCol = selCol;
 
@@ -973,7 +970,9 @@ void QColorShowLabel::mousePressEvent(QMouseEvent *e)
 
 void QColorShowLabel::mouseMoveEvent(QMouseEvent *e)
 {
-#ifndef QT_NO_DRAGANDDROP
+#ifdef QT_NO_DRAGANDDROP
+    Q_UNUSED(e);
+#else
     if (!mousePressed)
         return;
     if ((pressPos - e->pos()).manhattanLength() > QApplication::startDragDistance()) {
@@ -1213,12 +1212,12 @@ public:
     void setCurrentAlpha(int a) { cs->setCurrentAlpha(a); }
     void showAlpha(bool b) { cs->showAlpha(b); }
 
-    void addCustom();
+    void _q_addCustom();
 
-    void newHsv(int h, int s, int v);
-    void newColorTypedIn(QRgb rgb);
-    void newCustom(int, int);
-    void newStandard(int, int);
+    void _q_newHsv(int h, int s, int v);
+    void _q_newColorTypedIn(QRgb rgb);
+    void _q_newCustom(int, int);
+    void _q_newStandard(int, int);
 
     QWellArray *custom;
     QWellArray *standard;
@@ -1231,7 +1230,7 @@ public:
 };
 
 //sets all widgets to display h,s,v
-void QColorDialogPrivate::newHsv(int h, int s, int v)
+void QColorDialogPrivate::_q_newHsv(int h, int s, int v)
 {
     cs->setHsv(h, s, v);
     cp->setCol(h, s);
@@ -1242,11 +1241,11 @@ void QColorDialogPrivate::newHsv(int h, int s, int v)
 void QColorDialogPrivate::setCurrentColor(QRgb rgb)
 {
     cs->setRgb(rgb);
-    newColorTypedIn(rgb);
+    _q_newColorTypedIn(rgb);
 }
 
 //sets all widgets exept cs to display rgb
-void QColorDialogPrivate::newColorTypedIn(QRgb rgb)
+void QColorDialogPrivate::_q_newColorTypedIn(QRgb rgb)
 {
     int h, s, v;
     rgb2hsv(rgb, h, s, v);
@@ -1254,7 +1253,7 @@ void QColorDialogPrivate::newColorTypedIn(QRgb rgb)
     lp->setCol(h, s, v);
 }
 
-void QColorDialogPrivate::newCustom(int r, int c)
+void QColorDialogPrivate::_q_newCustom(int r, int c)
 {
     int i = r+2*c;
     setCurrentColor(cusrgb[i]);
@@ -1263,7 +1262,7 @@ void QColorDialogPrivate::newCustom(int r, int c)
         standard->setSelected(-1,-1);
 }
 
-void QColorDialogPrivate::newStandard(int r, int c)
+void QColorDialogPrivate::_q_newStandard(int r, int c)
 {
     setCurrentColor(stdrgb[r+c*6]);
     if (custom)
@@ -1297,7 +1296,7 @@ void QColorDialogPrivate::init()
 #ifndef QT_NO_SHORTCUT
         lab->setBuddy(standard);
 #endif
-        q->connect(standard, SIGNAL(selected(int,int)), SLOT(newStandard(int,int)));
+        q->connect(standard, SIGNAL(selected(int,int)), SLOT(_q_newStandard(int,int)));
         leftLay->addWidget(lab);
         leftLay->addWidget(standard);
 
@@ -1307,7 +1306,7 @@ void QColorDialogPrivate::init()
         custom = new QColorWell(q, 2, 8, cusrgb);
         custom->setAcceptDrops(true);
 
-        q->connect(custom, SIGNAL(selected(int,int)), SLOT(newCustom(int,int)));
+        q->connect(custom, SIGNAL(selected(int,int)), SLOT(_q_newCustom(int,int)));
         lab = new QLabel(QColorDialog::tr("&Custom colors") , q);
 #ifndef QT_NO_SHORTCUT
         lab->setBuddy(custom);
@@ -1346,12 +1345,12 @@ void QColorDialogPrivate::init()
     pickLay->addWidget(lp);
 
     QObject::connect(cp, SIGNAL(newCol(int,int)), lp, SLOT(setCol(int,int)));
-    QObject::connect(lp, SIGNAL(newHsv(int,int,int)), q, SLOT(newHsv(int,int,int)));
+    QObject::connect(lp, SIGNAL(newHsv(int,int,int)), q, SLOT(_q_newHsv(int,int,int)));
 
     rightLay->addStretch();
 
     cs = new QColorShower(q);
-    QObject::connect(cs, SIGNAL(newCol(QRgb)), q, SLOT(newColorTypedIn(QRgb)));
+    QObject::connect(cs, SIGNAL(newCol(QRgb)), q, SLOT(_q_newColorTypedIn(QRgb)));
     rightLay->addWidget(cs);
 
     QHBoxLayout *buttons;
@@ -1376,11 +1375,11 @@ void QColorDialogPrivate::init()
     if (!compact) {
         QPushButton *addCusBt = new QPushButton(QColorDialog::tr("&Add to Custom Colors"), q);
         rightLay->addWidget(addCusBt);
-        QObject::connect(addCusBt, SIGNAL(clicked()), q, SLOT(addCustom()));
+        QObject::connect(addCusBt, SIGNAL(clicked()), q, SLOT(_q_addCustom()));
     }
 }
 
-void QColorDialogPrivate::addCustom()
+void QColorDialogPrivate::_q_addCustom()
 {
     cusrgb[nextCust] = cs->currentColor();
     if (custom)
@@ -1615,7 +1614,7 @@ bool QColorDialog::selectColor(const QColor& col)
         for (i = 0; i < 6; i++) {
             for (j = 0; j < 8; j++) {
                 if (color == stdrgb[i + j*6]) {
-                    d->newStandard(i, j);
+                    d->_q_newStandard(i, j);
                     d->standard->setCurrent(i, j);
                     d->standard->setSelected(i, j);
                     d->standard->setFocus();
@@ -1629,7 +1628,7 @@ bool QColorDialog::selectColor(const QColor& col)
         for (i = 0; i < 2; i++) {
             for (j = 0; j < 8; j++) {
                 if (color == cusrgb[i + j*2]) {
-                    d->newCustom(i, j);
+                    d->_q_newCustom(i, j);
                     d->custom->setCurrent(i, j);
                     d->custom->setSelected(i, j);
                     d->custom->setFocus();
