@@ -635,7 +635,10 @@ QKeyEvent::~QKeyEvent()
     Returns the code of the key that was pressed or released.
 
     See \l Qt::Key for the list of keyboard codes. These codes are
-    independent of the underlying window system.
+    independent of the underlying window system. Note that this
+    function does not distinguish between capital and non-capital
+    letters, use the text() function (returning the Unicode text the
+    key generated) for this purpose.
 
     A value of either 0 or Qt::Key_unknown means that the event is not
     the result of a known key; for example, it may be the result of
@@ -2408,17 +2411,49 @@ QHelpEvent::~QHelpEvent()
     \ingroup events
     \ingroup helpsystem
 
-    Status tips can be set on a widget using QWidget::setStatusTip().
-    They are shown in the status bar when the mouse cursor enters the
-    widget. Status tips can also be set on actions using
-    QAction::setStatusTip(), and they are supported for the item view
-    classes through Qt::StatusTipRole.
+    Status tips can be set on a widget using the
+    QWidget::setStatusTip() function.  They are shown in the status
+    bar when the mouse cursor enters the widget. For example:
+
+    \table 100%
+    \row
+    \o
+    \quotefromfile snippets/qstatustipevent/main.cpp
+    \skipto MainWindow::MainWindow
+    \printuntil setCentralWidget
+    \dots
+    \skipto }
+    \printline }
+    \o
+    \image qstatustipevent-widget.png Widget with status tip.
+    \endtable
+
+    Status tips can also be set on actions using the
+    QAction::setStatusTip() function:
+
+    \table 100%
+    \row
+    \o
+    \quotefromfile snippets/qstatustipevent/main.cpp
+    \skipto MainWindow::MainWindow
+    \printuntil {
+    \skipto QMenu
+    \printuntil addAction
+    \dots
+    \skipto }
+    \printline }
+    \o
+    \image qstatustipevent-action.png Action with status tip.
+    \endtable
+
+    Finally, status tips are supported for the item view classes
+    through the Qt::StatusTipRole enum value.
 
     \sa QStatusBar, QHelpEvent, QWhatsThisClickedEvent
 */
 
 /*!
-    Constructs a status tip event with text specified by \a tip.
+    Constructs a status tip event with the text specified by \a tip.
 
     \sa tip()
 */
