@@ -41,6 +41,8 @@
 #include <QtGui/qfontmetrics.h>
 #endif
 
+QT_BEGIN_HEADER
+
 QT_MODULE(Gui)
 
 class QBrush;
@@ -801,4 +803,6 @@ inline void QPainter::drawPicture(const QPoint &pt, const QPicture &p)
 }
 #endif
 
+QT_END_HEADER
+
 #endif // QPAINTER_H
