@@ -29,6 +29,8 @@
 #ifdef QT3_SUPPORT
 #include <QtGui/qaction.h>
 
+QT_BEGIN_HEADER
+
 QT_MODULE(Gui)
 
 class Q_GUI_EXPORT QMenuItem : public QAction
@@ -44,6 +46,9 @@ private:
     void setId(int);
     void setSignalValue(int);
 };
+
+QT_END_HEADER
+
 #endif
 
 #endif // QMENUDATA_H
