@@ -40,6 +40,7 @@
 #include <qvariant.h>
 #include <qstylepainter.h>
 #include <private/qabstractbutton_p.h>
+#include <private/qaction_p.h>
 
 class QToolButtonPrivate : public QAbstractButtonPrivate
 {
@@ -47,10 +48,10 @@ class QToolButtonPrivate : public QAbstractButtonPrivate
 public:
     void init();
 #ifndef QT_NO_MENU
-    void buttonPressed();
+    void _q_buttonPressed();
     void popupTimerDone();
 #endif
-    void actionTriggered();
+    void _q_actionTriggered();
     QStyleOptionToolButton getStyleOption() const;
     QPointer<QAction> menuAction; //the menu set by the user (setMenu)
     QBasicTimer popupTimer;
@@ -88,9 +89,17 @@ bool QToolButtonPrivate::hasMenu() const
     A tool button is a special button that provides quick-access to
     specific commands or options. As opposed to a normal command
     button, a tool button usually doesn't show a text label, but shows
-    an icon instead. Its classic usage is to select tools, for example
+    an icon instead.
+
+    Tool buttons are normally created when new QAction instances are
+    created with QToolBar::addAction() or existing actions are added
+    to a toolbar with QToolBar::addAction(). It is also possible to
+    construct tool buttons in the same way as any other widget, and
+    arrange them alongside other widgets in layouts.
+
+    One classic use of a tool button is to select tools; for example,
     the "pen" tool in a drawing program. This would be implemented
-    with a QToolButton as toggle button (see setToggleButton()).
+    by using a QToolButton as a toggle button (see setToggleButton()).
 
     QToolButton supports auto-raising. In auto-raise mode, the button
     draws a 3D frame only when the mouse points at it. The feature is
@@ -119,10 +128,14 @@ bool QToolButtonPrivate::hasMenu() const
     of possible pages to jump to. The default delay is 600ms; you can
     adjust it with setPopupDelay().
 
-    \img qdockwindow.png Toolbar with Toolbuttons \caption A floating
-    QToolbar with QToolbuttons
+    \table 100%
+    \row \o \inlineimage assistant-toolbar1.png Qt Assistant's toolbar with tool buttons
+    \row \o Qt Assistant's toolbar contains tool buttons that are associated
+         with actions used in other parts of the main window.
+    \endtable
 
-    \sa QPushButton, QToolBar, QMainWindow, {fowler}{GUI Design Handbook: Push Button}
+    \sa QPushButton, QToolBar, QMainWindow, QAction,
+        {fowler}{GUI Design Handbook: Push Button}
 */
 
 /*!
@@ -237,7 +250,7 @@ void QToolButtonPrivate::init()
     q->setFocusPolicy(Qt::TabFocus);
     q->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
 
-    QObject::connect(q, SIGNAL(pressed()), q, SLOT(buttonPressed()));
+    QObject::connect(q, SIGNAL(pressed()), q, SLOT(_q_buttonPressed()));
 }
 
 
@@ -471,7 +484,7 @@ void QToolButton::actionEvent(QActionEvent *event)
             setDefaultAction(action); // update button state
         break;
     case QEvent::ActionAdded:
-        connect(action, SIGNAL(triggered()), this, SLOT(actionTriggered()));
+        connect(action, SIGNAL(triggered()), this, SLOT(_q_actionTriggered()));
         break;
     case QEvent::ActionRemoved:
         if (d->defaultAction == action)
@@ -488,7 +501,7 @@ void QToolButton::actionEvent(QActionEvent *event)
     QAbstractButton::actionEvent(event);
 }
 
-void QToolButtonPrivate::actionTriggered()
+void QToolButtonPrivate::_q_actionTriggered()
 {
     Q_Q(QToolButton);
     if (QAction *action = qobject_cast<QAction *>(q->sender()))
@@ -707,7 +720,7 @@ void QToolButton::showMenu()
     d->popupTimerDone();
 }
 
-void QToolButtonPrivate::buttonPressed()
+void QToolButtonPrivate::_q_buttonPressed()
 {
     Q_Q(QToolButton);
     if (!hasMenu())
@@ -900,8 +913,8 @@ bool QToolButton::autoRaise() const
 void QToolButton::setDefaultAction(QAction *action)
 {
     Q_D(QToolButton);
-    bool hadMenu = false;
 #ifndef QT_NO_MENU
+    bool hadMenu = false;
     hadMenu = d->hasMenu();
 #endif
     d->defaultAction = action;
@@ -931,7 +944,8 @@ void QToolButton::setDefaultAction(QAction *action)
     setCheckable(action->isCheckable());
     setChecked(action->isChecked());
     setEnabled(action->isEnabled());
-    setFont(action->font());
+    if (action->d_func()->fontSet)
+        setFont(action->font());
 }
 
 
