@@ -26,6 +26,8 @@
 
 #include <QtNetwork/qabstractsocket.h>
 
+QT_BEGIN_HEADER
+
 QT_MODULE(Network)
 
 class QTcpSocketPrivate;
@@ -42,4 +44,6 @@ private:
     Q_DECLARE_PRIVATE(QTcpSocket)
 };
 
+QT_END_HEADER
+
 #endif // QTCPSOCKET_H
