@@ -26,6 +26,8 @@
 
 #include <Qt3Support/q3scrollview.h>
 
+QT_BEGIN_HEADER
+
 QT_MODULE(Qt3SupportLight)
 
 #ifndef QT_NO_LISTVIEW
@@ -580,4 +582,6 @@ private:
 
 #endif // QT_NO_LISTVIEW
 
+QT_END_HEADER
+
 #endif // Q3LISTVIEW_H
