@@ -129,6 +129,8 @@ static QStyleOptionHeader getStyleOption(const Q3Header *header, int section)
     QStyleOptionHeader opt;
     opt.init(header);
     opt.section = section;
+    opt.textAlignment = Qt::AlignVCenter;
+    opt.iconAlignment = Qt::AlignVCenter;
     if (header->iconSet(section))
         opt.icon = *header->iconSet(section);
     opt.text = header->label(section);
@@ -137,6 +139,16 @@ static QStyleOptionHeader getStyleOption(const Q3Header *header, int section)
     return opt;
 }
 
+bool qt_get_null_label_bit(Q3HeaderData *data, int section)
+{
+    return data->nullStringLabels.testBit(section);
+}
+
+void qt_set_null_label_bit(Q3HeaderData *data, int section, bool b)
+{
+    data->nullStringLabels.setBit(section, b);
+}
+
 /*!
     \class Q3Header q3header.h
     \brief The Q3Header class provides a header row or column, e.g. for
