@@ -26,6 +26,8 @@
 
 #include <Qt3Support/q3hbox.h>
 
+QT_BEGIN_HEADER
+
 QT_MODULE(Qt3SupportLight)
 
 class Q_COMPAT_EXPORT Q3VBox : public Q3HBox
@@ -38,4 +40,6 @@ private:
     Q_DISABLE_COPY(Q3VBox)
 };
 
+QT_END_HEADER
+
 #endif // Q3VBOX_H
