@@ -30,10 +30,8 @@ class GammaView: public QWidget
 {
     Q_OBJECT
 public:
-    GammaView( QWidget *parent = 0,
-		const char *name = 0, Qt::WFlags f = 0 ) :
-	QWidget(parent,name,f)
-    { }
+    GammaView( QWidget *parent = 0, Qt::WFlags f = 0 )
+        : QWidget(parent,f) { }
 };
 
 #endif
