@@ -33,8 +33,7 @@ class QVFbRateDialog : public QDialog
 {
     Q_OBJECT
 public:
-    QVFbRateDialog( int value, QWidget *parent=0, const char *name=0,
-		    bool modal=FALSE );
+    QVFbRateDialog( int value, QWidget *parent=0 );
 
 signals:
     void updateRate( int r );
